@@ -65,11 +65,7 @@ class MySQL_Connection {
 	unsigned long long last_time_used;
 	unsigned long long timeout;
 	int fd;
-#ifndef PROXYSQL_STMT_V14
-	MySQL_STMTs_local *local_stmts;	// local view of prepared statements
-#else
 	MySQL_STMTs_local_v14 *local_stmts;	// local view of prepared statements
-#endif
 	MYSQL *mysql;
 	MYSQL *ret_mysql;
 	MYSQL_RES *mysql_result;
@@ -95,6 +91,8 @@ class MySQL_Connection {
 	bool processing_prepared_statement_execute;
 	bool processing_multi_statement;
 	bool multiplex_delayed;
+	bool unknown_transaction_status;
+	void compute_unknown_transaction_status();
 	MySQL_Connection();
 	~MySQL_Connection();
 	bool set_autocommit(bool);
