@@ -612,6 +612,7 @@ __thread int mysql_thread___ping_timeout_server;
 __thread int mysql_thread___shun_on_failures;
 __thread int mysql_thread___shun_recovery_time_sec;
 __thread int mysql_thread___query_retries_on_failure;
+__thread bool mysql_thread___client_multi_statements;
 __thread int mysql_thread___connect_retries_on_failure;
 __thread int mysql_thread___connect_retries_delay;
 __thread int mysql_thread___connection_delay_multiplex_ms;
@@ -718,6 +719,7 @@ extern __thread int mysql_thread___ping_timeout_server;
 extern __thread int mysql_thread___shun_on_failures;
 extern __thread int mysql_thread___shun_recovery_time_sec;
 extern __thread int mysql_thread___query_retries_on_failure;
+extern __thread bool mysql_thread___client_multi_statements;
 extern __thread int mysql_thread___connect_retries_on_failure;
 extern __thread int mysql_thread___connect_retries_delay;
 extern __thread int mysql_thread___connection_delay_multiplex_ms;
