@@ -3,7 +3,6 @@
 
 #include "SpookyV2.h"
 
-//#ifndef PROXYSQL_STMT_V14
 //extern MySQL_STMT_Manager *GloMyStmt;
 //static uint32_t add_prepared_statement_calls = 0;
 //static uint32_t find_prepared_statement_by_hash_calls = 0;
@@ -133,14 +132,11 @@ void *StmtLongDataHandler::get(uint32_t _stmt_id, uint16_t _param_id,
 	return NULL;
 }
 
-//#ifndef PROXYSQL_STMT_V14
-//MySQL_STMT_Global_info::MySQL_STMT_Global_info(uint32_t id, unsigned int h,
-//#else
 MySQL_STMT_Global_info::MySQL_STMT_Global_info(uint64_t id, unsigned int h,
-//#endif
                                                char *u, char *s, char *q,
                                                unsigned int ql,
                                                MYSQL_STMT *stmt, uint64_t _h) {
+	pthread_rwlock_init(&rwlock_, NULL);
 	statement_id = id;
 	hostgroup_id = h;
 	ref_count_client = 0;
@@ -152,6 +148,7 @@ MySQL_STMT_Global_info::MySQL_STMT_Global_info(uint64_t id, unsigned int h,
 	memcpy(query, q, ql);
 	query[ql] = '\0';  // add NULL byte
 	query_length = ql;
+	MyComQueryCmd = MYSQL_COM_QUERY__UNINITIALIZED;
 	num_params = stmt->param_count;
 	num_columns = stmt->field_count;
 	warning_count = stmt->upsert_status.warning_count;
@@ -223,10 +220,10 @@ MySQL_STMT_Global_info::MySQL_STMT_Global_info(uint64_t id, unsigned int h,
 	}
 }
 
-#ifdef PROXYSQL_STMT_V14
 void MySQL_STMT_Global_info::update_metadata(MYSQL_STMT *stmt) {
 	int i;
 	bool need_refresh = false;
+	pthread_rwlock_wrlock(&rwlock_);
 	if (
 		(num_params != stmt->param_count)
 		||
@@ -406,8 +403,8 @@ void MySQL_STMT_Global_info::update_metadata(MYSQL_STMT *stmt) {
 		}
 // till here is copied from constructor
 	}
+	pthread_rwlock_unlock(&rwlock_);
 }
-#endif
 
 MySQL_STMT_Global_info::~MySQL_STMT_Global_info() {
 	free(username);
@@ -464,300 +461,7 @@ MySQL_STMT_Global_info::~MySQL_STMT_Global_info() {
 		digest_text = NULL;
 	}
 }
-/*
-#ifndef PROXYSQL_STMT_V14
-uint64_t MySQL_STMTs_local::compute_hash(unsigned int hostgroup, char *user,
-                                         char *schema, char *query,
-                                         unsigned int query_length) {
-	uint64_t hash;
-	hash = stmt_compute_hash(hostgroup, user, schema, query, query_length);
-	return hash;
-}
 
-MySQL_STMTs_local::~MySQL_STMTs_local() {
-	// Note: we do not free the prepared statements because we assume that
-	// if we call this destructor the connection is being destroyed anyway
-	for (std::map<uint32_t, MYSQL_STMT *>::iterator it = m.begin();
-	     it != m.end(); ++it) {
-		uint32_t stmt_id = it->first;
-		MYSQL_STMT *stmt = it->second;
-		if (stmt) {  // is a server
-			if (stmt->mysql) {
-				stmt->mysql->stmts =
-				    list_delete(stmt->mysql->stmts, &stmt->list);
-			}
-			// we do a hack here: we pretend there is no server associate
-			// the connection will be dropped anyway immediately after
-			stmt->mysql = NULL;
-			mysql_stmt_close(stmt);
-			GloMyStmt->ref_count(stmt_id, -1, true, false);
-		} else {  // is a client
-			GloMyStmt->ref_count(stmt_id, -1, true, true);
-		}
-	}
-	m.erase(m.begin(), m.end());
-}
-
-uint32_t MySQL_STMTs_local::generate_new_stmt_id(uint32_t global_statement_id) {
-	uint32_t new_id;
-	new_id = GloMyStmt->generate_new_stmt_id();
-	client_stmt_to_global_id.insert(std::make_pair(global_statement_id, new_id));
-	return new_id;
-}
-
-
-uint32_t MySQL_STMTs_local::find_original_id(uint32_t client_stmt_id) {
-	auto s = client_stmt_to_global_id.find(client_stmt_id);
-	if (s != client_stmt_to_global_id.end()) {
-		uint32_t ret=s->second;
-		return ret;
-	}
-	return 0;
-}
-
-bool MySQL_STMTs_local::erase(uint32_t global_statement_id) {
-	auto s = m.find(global_statement_id);
-	if (s != m.end()) {  // found
-		if (is_client) {
-			// we are removing it from a client, not backend
-			GloMyStmt->ref_count(global_statement_id, -1, true, true);
-			m.erase(s);
-			return true;
-		}
-		// the following seems deprecated for now. Asserting
-		assert(0);
-		if (num_entries > 1000) {
-			MYSQL_STMT *stmt = s->second;
-			mysql_stmt_close(stmt);
-			m.erase(s);
-			num_entries--;
-			return true;  // we truly removed the prepared statement
-		}
-	}
-	return false;  // we don't really remove the prepared statement
-}
-
-void MySQL_STMTs_local::insert(uint32_t global_statement_id, MYSQL_STMT *stmt) {
-	std::pair<std::map<uint32_t, MYSQL_STMT *>::iterator, bool> ret;
-	ret = m.insert(std::make_pair(global_statement_id, stmt));
-	if (ret.second == true) {
-		num_entries++;
-	}
-	if (stmt == NULL) {  // only for clients
-		GloMyStmt->ref_count(global_statement_id, 1, true, true);
-	}
-}
-
-MySQL_STMT_Manager::MySQL_STMT_Manager() {
-	spinlock_rwlock_init(&rwlock);
-	next_statement_id =
-		33;	// we initialize this as 33, leaving the first 32 reserved for special prepared statements (future use)
-}
-
-MySQL_STMT_Manager::~MySQL_STMT_Manager() {
-	for (std::map<uint32_t, MySQL_STMT_Global_info *>::iterator it = m.begin();
-	     it != m.end(); ++it) {
-		MySQL_STMT_Global_info *a = it->second;
-		delete a;
-	}
-	m.erase(m.begin(), m.end());
-	// we do not loop in h because all the MySQL_STMT_Global_info() were already
-	// deleted
-	h.erase(h.begin(), h.end());
-}
-
-void MySQL_STMT_Manager::active_prepared_statements(uint32_t *unique,
-                                                    uint32_t *total) {
-	uint32_t u = 0;
-	uint32_t t = 0;
-	spin_wrlock(&rwlock);
-	// fprintf(stderr,"%u , %u , %u , %u\n",
-	// find_prepared_statement_by_hash_calls, add_prepared_statement_calls,
-	// m.size(), total_prepared_statements());
-	for (std::map<uint32_t, MySQL_STMT_Global_info *>::iterator it = m.begin();
-	     it != m.end(); ++it) {
-		MySQL_STMT_Global_info *a = it->second;
-		if (a->ref_count_client) {
-			u++;
-			t += a->ref_count_client;
-#ifdef DEBUG
-			fprintf(stderr, "stmt %d , client_ref_count %d\n", a->statement_id,
-			        a->ref_count_client);
-#endif
-		}
-	}
-	spin_wrunlock(&rwlock);
-	*unique = u;
-	*total = t;
-}
-
-int MySQL_STMT_Manager::ref_count(uint32_t statement_id, int cnt, bool lock,
-                                  bool is_client) {
-	int ret = -1;
-	if (lock) {
-		spin_wrlock(&rwlock);
-	}
-	auto s = m.find(statement_id);
-	if (s != m.end()) {
-		MySQL_STMT_Global_info *a = s->second;
-		if (is_client) {
-			ret = __sync_add_and_fetch(&a->ref_count_client, cnt);
-			//__sync_fetch_and_add(&a->ref_count_client,cnt);
-			// ret=a->ref_count_client;
-			if (m.size() > (unsigned)mysql_thread___max_stmts_cache) {
-				int max_purge = m.size() / 20;  // purge up to 5%
-				int i = -1;
-				uint32_t *torem =
-				    (uint32_t *)malloc(max_purge * sizeof(uint32_t));
-				for (std::map<uint32_t, MySQL_STMT_Global_info *>::iterator it =
-				         m.begin();
-				     it != m.end(); ++it) {
-					if (i == (max_purge - 1)) continue;
-					MySQL_STMT_Global_info *a = it->second;
-					if (__sync_add_and_fetch(&a->ref_count_client, 0) == 0) {
-						uint64_t hash = a->hash;
-						auto s2 = h.find(hash);
-						if (s2 != h.end()) {
-							h.erase(s2);
-						}
-						// m.erase(it);
-						// delete a;
-						i++;
-						torem[i] = it->first;
-					}
-				}
-				while (i >= 0) {
-					uint32_t id = torem[i];
-					auto s3 = m.find(id);
-					MySQL_STMT_Global_info *a = s3->second;
-					if (a->ref_count_server == 0) {
-						free_stmt_ids.push(id);
-					}
-					m.erase(s3);
-					delete a;
-					i--;
-				}
-				free(torem);
-			}
-		} else {
-			__sync_fetch_and_add(&a->ref_count_server, cnt);
-			ret = a->ref_count_server;
-		}
-	}
-	if (lock) {
-		spin_wrunlock(&rwlock);
-	}
-	return ret;
-}
-
-MySQL_STMT_Global_info *MySQL_STMT_Manager::add_prepared_statement(
-    bool *is_new, unsigned int _h, char *u, char *s, char *q, unsigned int ql,
-    MYSQL_STMT *stmt, bool lock) {
-	return add_prepared_statement(is_new, _h, u, s, q, ql, stmt, -1, -1, -1,
-	                              lock);
-}
-
-MySQL_STMT_Global_info *MySQL_STMT_Manager::add_prepared_statement(
-    bool *is_new, unsigned int _h, char *u, char *s, char *q, unsigned int ql,
-    MYSQL_STMT *stmt, int _cache_ttl, int _timeout, int _delay, bool lock) {
-	MySQL_STMT_Global_info *ret = NULL;
-	uint64_t hash = stmt_compute_hash(
-	    _h, u, s, q, ql);  // this identifies the prepared statement
-	if (lock) {
-		spin_wrlock(&rwlock);
-	}
-	// try to find the statement
-	auto f = h.find(hash);
-	if (f != h.end()) {
-		// found it!
-		// MySQL_STMT_Global_info *a=f->second;
-		// ret=a->statement_id;
-		ret = f->second;
-		*is_new = false;
-	} else {
-		// we need to create a new one
-		bool free_id_avail = false;
-		free_id_avail = free_stmt_ids.size();
-		uint32_t next_id = 0;
-		if (free_id_avail) {
-			next_id = free_stmt_ids.top();
-			free_stmt_ids.pop();
-		} else {
-			// next_id = next_statement_id;
-			// next_statement_id++;
-			__sync_fetch_and_add(&next_statement_id, 1);
-		}
-		MySQL_STMT_Global_info *a =
-		    new MySQL_STMT_Global_info(next_id, _h, u, s, q, ql, stmt, hash);
-		a->properties.cache_ttl = _cache_ttl;
-		a->properties.timeout = _timeout;
-		a->properties.delay = _delay;
-		// insert it in both maps
-		m.insert(std::make_pair(a->statement_id, a));
-		h.insert(std::make_pair(a->hash, a));
-		// ret=a->statement_id;
-		ret = a;
-		// next_statement_id++;	// increment it
-		//__sync_fetch_and_add(&ret->ref_count_client,1); // increase reference
-		//count
-		__sync_fetch_and_add(&ret->ref_count_client,
-		                     1);  // increase reference count
-		*is_new = true;
-	}
-	__sync_fetch_and_add(&add_prepared_statement_calls, 1);
-	__sync_fetch_and_add(&ret->ref_count_server,
-	                     1);  // increase reference count
-	if (lock) {
-		spin_wrunlock(&rwlock);
-	}
-	return ret;
-}
-
-MySQL_STMT_Global_info *MySQL_STMT_Manager::find_prepared_statement_by_stmt_id(
-    uint32_t id, bool lock) {
-	MySQL_STMT_Global_info *ret = NULL;  // assume we do not find it
-	if (lock) {
-		spin_wrlock(&rwlock);
-	}
-
-	auto s = m.find(id);
-	if (s != m.end()) {
-		ret = s->second;
-		//__sync_fetch_and_add(&ret->ref_count,1); // increase reference count
-	}
-
-	if (lock) {
-		spin_wrunlock(&rwlock);
-	}
-	return ret;
-}
-
-MySQL_STMT_Global_info *MySQL_STMT_Manager::find_prepared_statement_by_hash(
-    uint64_t hash, bool lock) {
-	MySQL_STMT_Global_info *ret = NULL;  // assume we do not find it
-	if (lock) {
-		spin_wrlock(&rwlock);
-	}
-
-	auto s = h.find(hash);
-	if (s != h.end()) {
-		ret = s->second;
-		//__sync_fetch_and_add(&ret->ref_count_client,1); // increase reference
-		//count
-		__sync_fetch_and_add(&find_prepared_statement_by_hash_calls, 1);
-		__sync_fetch_and_add(&ret->ref_count_client, 1);
-	}
-
-	if (lock) {
-		spin_wrunlock(&rwlock);
-	}
-	return ret;
-}
-
-
-
-#else // PROXYSQL_STMT_V14
-*/
 extern MySQL_STMT_Manager_v14 *GloMyStmt;
 
 void MySQL_STMTs_local_v14::backend_insert(uint64_t global_statement_id, MYSQL_STMT *stmt) {
@@ -779,6 +483,7 @@ uint64_t MySQL_STMTs_local_v14::compute_hash(unsigned int hostgroup, char *user,
 }
 
 MySQL_STMT_Manager_v14::MySQL_STMT_Manager_v14() {
+	last_purge_time = time(NULL);
 	pthread_rwlock_init(&rwlock_, NULL);
         map_stmt_id_to_info= std::map<uint64_t, MySQL_STMT_Global_info *>();       // map using statement id
         map_stmt_hash_to_info = std::map<uint64_t, MySQL_STMT_Global_info *>();     // map using hashes
@@ -786,6 +491,14 @@ MySQL_STMT_Manager_v14::MySQL_STMT_Manager_v14() {
 
 	next_statement_id =
 	    1;  // we initialize this as 1 because we 0 is not allowed
+	num_stmt_with_ref_client_count_zero = 0;
+	num_stmt_with_ref_server_count_zero = 0;
+	statuses.c_unique = 0;
+	statuses.c_total = 0;
+	statuses.stmt_max_stmt_id = 0;
+	statuses.cached = 0;
+	statuses.s_unique = 0;
+	statuses.s_total = 0;
 }
 
 MySQL_STMT_Manager_v14::~MySQL_STMT_Manager_v14() {
@@ -796,24 +509,51 @@ void MySQL_STMT_Manager_v14::ref_count_client(uint64_t _stmt_id ,int _v, bool lo
 		pthread_rwlock_wrlock(&rwlock_);
 	auto s = map_stmt_id_to_info.find(_stmt_id);
 	if (s != map_stmt_id_to_info.end()) {
+		statuses.c_total += _v;
 		MySQL_STMT_Global_info *stmt_info = s->second;
+		if (stmt_info->ref_count_client == 0 && _v == 1) {
+			__sync_sub_and_fetch(&num_stmt_with_ref_client_count_zero,1);
+		} else {
+			if (stmt_info->ref_count_client == 1 && _v == -1) {
+				__sync_add_and_fetch(&num_stmt_with_ref_client_count_zero,1);
+			}
+		}
 		stmt_info->ref_count_client += _v;
-			if (map_stmt_id_to_info.size() > (unsigned)mysql_thread___max_stmts_cache) {
-				int max_purge = map_stmt_id_to_info.size() / 20;  // purge up to 5%
+			time_t ct = time(NULL);
+			uint64_t num_client_count_zero = __sync_add_and_fetch(&num_stmt_with_ref_client_count_zero, 0);
+			uint64_t num_server_count_zero = __sync_add_and_fetch(&num_stmt_with_ref_server_count_zero, 0);
+
+			size_t map_size = map_stmt_id_to_info.size();
+			if (
+				(ct > last_purge_time+1) &&
+				(map_size > (unsigned)mysql_thread___max_stmts_cache ) &&
+				(num_client_count_zero > map_size/10) &&
+				(num_server_count_zero > map_size/10)
+			) { // purge only if there is at least 10% gain
+				last_purge_time = ct;
+				int max_purge = map_size ;
 				int i = -1;
 				uint64_t *torem =
 				    (uint64_t *)malloc(max_purge * sizeof(uint64_t));
 				for (std::map<uint64_t, MySQL_STMT_Global_info *>::iterator it =
 				         map_stmt_id_to_info.begin();
 				     it != map_stmt_id_to_info.end(); ++it) {
-					if (i == (max_purge - 1)) continue;
+					if ( (i == (max_purge - 1)) || (i == (num_client_count_zero - 1)) ) {
+						break; // nothing left to clean up
+					}
 					MySQL_STMT_Global_info *a = it->second;
-					if (__sync_add_and_fetch(&a->ref_count_client, 0) == 0) {
+					if ((__sync_add_and_fetch(&a->ref_count_client, 0) == 0) &&
+						(a->ref_count_server == 0) ) // this to avoid that IDs are incorrectly reused
+					{
 						uint64_t hash = a->hash;
 						auto s2 = map_stmt_hash_to_info.find(hash);
 						if (s2 != map_stmt_hash_to_info.end()) {
 							map_stmt_hash_to_info.erase(s2);
 						}
+						__sync_sub_and_fetch(&num_stmt_with_ref_client_count_zero,1);
+						//if (a->ref_count_server == 0) {
+							//__sync_sub_and_fetch(&num_stmt_with_ref_server_count_zero,1);
+						//}
 						// m.erase(it);
 						// delete a;
 						i++;
@@ -825,9 +565,11 @@ void MySQL_STMT_Manager_v14::ref_count_client(uint64_t _stmt_id ,int _v, bool lo
 					auto s3 = map_stmt_id_to_info.find(id);
 					MySQL_STMT_Global_info *a = s3->second;
 					if (a->ref_count_server == 0) {
+						__sync_sub_and_fetch(&num_stmt_with_ref_server_count_zero,1);
 						free_stmt_ids.push(id);
 					}
 					map_stmt_id_to_info.erase(s3);
+					statuses.s_total -= a->ref_count_server;
 					delete a;
 					i--;
 				}
@@ -844,7 +586,15 @@ void MySQL_STMT_Manager_v14::ref_count_server(uint64_t _stmt_id ,int _v, bool lo
 	std::map<uint64_t, MySQL_STMT_Global_info *>::iterator s;
 	s = map_stmt_id_to_info.find(_stmt_id);
 	if (s != map_stmt_id_to_info.end()) {
+		statuses.s_total += _v;
 		MySQL_STMT_Global_info *stmt_info = s->second;
+		if (stmt_info->ref_count_server == 0 && _v == 1) {
+			__sync_sub_and_fetch(&num_stmt_with_ref_server_count_zero,1);
+		} else {
+			if (stmt_info->ref_count_server == 1 && _v == -1) {
+				__sync_add_and_fetch(&num_stmt_with_ref_server_count_zero,1);
+			}
+		}
 		stmt_info->ref_count_server += _v;
 	}
 	if (lock)
@@ -1046,8 +796,14 @@ MySQL_STMT_Global_info *MySQL_STMT_Manager_v14::add_prepared_statement(
 //		__sync_fetch_and_add(&ret->ref_count_client,
 //		                     1);  // increase reference count
 //		*is_new = true;
+		__sync_add_and_fetch(&num_stmt_with_ref_client_count_zero,1);
+		__sync_add_and_fetch(&num_stmt_with_ref_server_count_zero,1);
+	}
+	if (ret->ref_count_server == 0) {
+		__sync_sub_and_fetch(&num_stmt_with_ref_server_count_zero,1);
 	}
 	ret->ref_count_server++;
+	statuses.s_total++;
 //	__sync_fetch_and_add(&add_prepared_statement_calls, 1);
 //	__sync_fetch_and_add(&ret->ref_count_server,
 //	                     1);  // increase reference count
@@ -1067,6 +823,10 @@ void MySQL_STMT_Manager_v14::get_metrics(uint64_t *c_unique, uint64_t *c_total,
 	uint64_t s_u = 0;
 	uint64_t s_t = 0;
 	pthread_rwlock_wrlock(&rwlock_);
+	statuses.cached = map_stmt_id_to_info.size();
+	statuses.c_unique = statuses.cached - num_stmt_with_ref_client_count_zero;
+	statuses.s_unique = statuses.cached - num_stmt_with_ref_server_count_zero;
+#ifdef DEBUG
 	for (std::map<uint64_t, MySQL_STMT_Global_info *>::iterator it = map_stmt_id_to_info.begin();
 	     it != map_stmt_id_to_info.end(); ++it) {
 		MySQL_STMT_Global_info *a = it->second;
@@ -1083,17 +843,22 @@ void MySQL_STMT_Manager_v14::get_metrics(uint64_t *c_unique, uint64_t *c_total,
 			m = it->first;
 		}
 	}
-	pthread_rwlock_unlock(&rwlock_);
-	*c_unique = c_u;
-	*c_total = c_t;
+	assert (c_u == statuses.c_unique);
+	assert (c_t == statuses.c_total);
+	assert (c == statuses.cached);
+	assert (s_t == statuses.s_total);
+	assert (s_u == statuses.s_unique);
 	*stmt_max_stmt_id = m;
-	*cached = c;
-	*s_unique = s_u;
-	*s_total = s_t;
+#endif
+	*stmt_max_stmt_id = next_statement_id; // this is max stmt_id, no matter if in used or not
+	*c_unique = statuses.c_unique;
+	*c_total = statuses.c_total;
+	*cached = statuses.cached;
+	*s_total = statuses.s_total;
+	*s_unique = statuses.s_unique;
+	pthread_rwlock_unlock(&rwlock_);
 }
 
-//#endif // PROXYSQL_STMT_V14
-
 
 class PS_global_stats {
 	public:
