@@ -201,14 +201,10 @@ Query_Processor *GloQPro;
 ProxySQL_Admin *GloAdmin;
 MySQL_Threads_Handler *GloMTH;
 
-#ifndef PROXYSQL_STMT_V14
-MySQL_STMT_Manager *GloMyStmt;
-#else
 MySQL_STMT_Manager_v14 *GloMyStmt;
-#endif
 
 MySQL_Monitor *GloMyMon;
-std::thread *MyMon_thread;
+std::thread *MyMon_thread = NULL;
 
 MySQL_Logger *GloMyLogger;
 
@@ -400,11 +396,7 @@ void ProxySQL_Main_init_main_modules() {
 	MyHGM=new MySQL_HostGroups_Manager();
 	GloMTH=new MySQL_Threads_Handler();
 	GloMyLogger = new MySQL_Logger();
-#ifndef PROXYSQL_STMT_V14
-	GloMyStmt=new MySQL_STMT_Manager();
-#else
 	GloMyStmt=new MySQL_STMT_Manager_v14();
-#endif
 }
 
 
@@ -505,7 +497,7 @@ void ProxySQL_Main_join_all_threads() {
 	}
 
 	// join GloMyMon thread
-	if (GloMyMon) {
+	if (GloMyMon && MyMon_thread) {
 		cpu_timer t;
 		MyMon_thread->join();
 #ifdef DEBUG
